import * as THREE from 'three';
import { CogniteClient } from '@cognite/sdk';
export { THREE };

export class Cognite3DViewer {
  static isBrowserSupported(): boolean;
  domElement: HTMLElement;
  constructor(options?: Cognite3DViewerOptions);

  dispose(): void;

  on(type: 'click', callback: (event: PointerEvent) => void): void;
  off(type: 'click', callback: (event: PointerEvent) => void): void;

  on(type: 'cameraChange', callback: (position: THREE.Vector3, target: THREE.Vector3) => void): void;
  off(type: 'cameraChange', callback: (position: THREE.Vector3, target: THREE.Vector3) => void): void;

  addModel(options: AddModelOptions): Promise<Cognite3DModel>;
  addObject3D(object: THREE.Object3D): void;
  removeObject3D(object: THREE.Object3D): void;

  setSlicingPlanes(slicingPlanes: THREE.Plane[]): void;

  getCameraPosition(): THREE.Vector3;
  getCameraTarget(): THREE.Vector3;
  setCameraPosition(position: THREE.Vector3): void;
  setCameraTarget(position: THREE.Vector3): void;

  fitCameraToModel(model: Cognite3DModel, duration?: number): void;
  fitCameraToBoundingBox(box: THREE.Box3, duration?: number, radiusFactor?: number): void;

  enableKeyboardNavigation(): void;
  disableKeyboardNavigation(): void;

  worldToScreen(point: THREE.Vector3, normalize?: boolean): THREE.Vector2 | null;

  getScreenshot(width?: number, height?: number): Promise<string>;
  getIntersectionFromPixel(x: number, y: number, cognite3DModel?: Cognite3DModel): null | Intersection;

  clearCache(): void;
}

export class Cognite3DModel extends THREE.Object3D {
  modelId: number;
  revisionId: number;
  getSubtreeNodeIds(nodeId: number, subtreeSize?: number): Promise<number[]>;
  getBoundingBox(nodeId?: number, box?: THREE.Box3): THREE.Box3;

  iterateNodes(action: (nodeId: number, treeIndex?: number) => void): void;
  iterateSubtree(
    nodeId: number,
    action: (nodeId: number, treeIndex?: number) => void,
    treeIndex?: number,
    subtreeSize?: number
  ): Promise<boolean>;

  getNodeColor(nodeId: number): Color;
  setNodeColor(nodeId: number, r: number, g: number, b: number): void;
  resetNodeColor(nodeId: number): void;

  selectNode(nodeId: number): void;
  deselectNode(nodeId: number): void;
  deselectAllNodes(): void;

  showNode(nodeId: number): void;
  showAllNodes(): void;
  hideAllNodes(makeGray?: boolean): void;
  hideNode(nodeId: number, makeGray?: boolean): void;
}

export interface Intersection {
  model: Cognite3DModel;
  nodeId: number;
  point: THREE.Vector3;
}

// import Color from '@/core/Cognite3DModel';

export interface Color {
  r: number;
  g: number;
  b: number;
}

export interface Cognite3DViewerOptions {
  domElement?: HTMLElement;
  noBackground?: boolean;
  logMetrics?: boolean;
  highlightColor?: THREE.Color;
  // Only if you need to access CDF models
  sdk?: CogniteClient;
  viewCube?: 'topleft' | 'topright' | 'bottomleft' | 'bottomright';
  enableCache: boolean;
}

// export { OnProgressData } from '@/helpers/Progress';
export interface OnProgressData {
  step: number;
  numSteps: number;
  title: string;
  progress: number;
  max: number;
}
export interface GeometryFilter {
  boundingBox?: THREE.Box3;
}

export interface AddModelOptions {
  modelId: number;
  revisionId: number;
  // if you need to access local files, this is where you would specify it
  localPath?: string;
  geometryFilter?: GeometryFilter;
  orthographicCamera?: boolean;
  onProgress?: (progress: OnProgressData) => void;
  onComplete?: () => void;
}

// import { AddModelOptions, GeometryFilter } from '@/loaders/Cognite3DModelLoader';
// export { AddModelOptions, GeometryFilter };
